/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.startstopp.prozessvew;

import static org.junit.Assert.*;

import java.util.Date;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.startstopp.skriptvew.StartStoppApp;
import de.bsvrz.sys.startstopp.skriptvew.XMLToolsTest;

/**
 * Klasse zum Testen des Startens des Kernsystems
 * Vorgehen des Tests:
 * - Versionieren einer XML-Datei in der das Kernsystem gestartet wird
 * - Starten von StartStopp mit dieser versionierten Datei
 * - Versuchen sich an dem gestarteten Kernsystem anzumelden
 * - Senden einer StartStoppAnfrage Version 4 zum Beenden des Kernsystems
 *   
 * @author Dambach Werke GmbH
 */
public class KernsystemTest
{
  /**
   * Datenverteilerverbindung
   */
  private static ClientDavInterface m_connection = null;

  /**
   * ID des gestarteten Prozesses (Variante 4)
   */
  private String m_rechnerId;
  
  /**
   * StartStopp Antwort empfangen
   */
  private boolean m_ok;

  /**
   * StartStopp Antwort empfangen
   */
  private boolean m_empfangen;

  /**
   * PID des Datenverteilerprozesses
   */
  private String m_pidDatenverteiler;
  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * Methode prft den Start des Kernsystems von der StartStopp Apllikation heraus und beendet das
   * Kernsystem ber Versenden der StartStoppAnfrage Variante 4
   */
  @Test
  public void testStarteKensystem ()
  {
    String xmlDatei  = "C:/prozessverwaltungTest.xml";
    
    // XML-String aus Resource einlesen und in Datei schreiben
    
    XMLToolsTest test = new XMLToolsTest ();
    String xml = test.einlesenAusResource( "../xml/startStopp_Kernsystem_2.xml" );
    test.schreibeInDatei( xmlDatei, xml );

    // Diese Datei versionieren
    
    String[] args = new String [3];
    
    args [0] = "-startStoppKonfiguration=C:/TestStartStopp"; 
    args [1] = "-versionieren=" + xmlDatei;
    args [2] = "-reset=true";

    StartStoppApp app = new StartStoppApp ();
    app.main( args );
    
    // StartStopp mit dieser Datei starten

    String[] args1 = new String [3];
    args1 [0] = "-startStoppKonfiguration=C:/TestStartStopp"; 
    args1 [1] = "-reset=true";
    args1 [2] = "-port=23";

    app = new StartStoppApp ();
    app.main( args1 );

    // Wartezeit damit StartStopp das Kernsystem starten kann
    
    mySleep (60000 );
    
    // Datenverteilerverbindung aufbauen zum Prfen der gesendeten Datenstze

    // Zugangsdaten DaV
    
    String rechner  = "localhost";
    int port        = 8083;
    String benutzer = "Tester";
    String password = "geheim";

    ClientDavParameters parameters = null;
    try
    {
      parameters = new ClientDavParameters();
      
      parameters.setDavCommunicationAddress( rechner );
      parameters.setDavCommunicationSubAddress( port );
      parameters.setUserName( benutzer );
      parameters.setUserPassword( password );

      m_connection = new ClientDavConnection(parameters);
      
      m_connection.connect();
      
      m_connection.login();
      
      System.out.println("*** Mit DAV verbunden ***");
    }
    catch (MissingParameterException e)
    {
      e.printStackTrace();
    }
    catch (CommunicationError e)
    {
      e.printStackTrace();
    }
    catch (ConnectionException e)
    {
      e.printStackTrace();
    }
    catch (InconsistentLoginException e)
    {
      e.printStackTrace();
    }

    // Von JUNIT aus am DaV anmelden und die empfangene Prozessinformationen auswerten
    
    final Object sync = new Object();

    m_empfangen = false;
    
    final String _atg1 = "atg.prozessInfo";
    final String _atg2 = "atg.startStoppInfo";

    m_empfangen  = false;
    m_ok         = false;

    // Neuen Empfnger anlegen der auf die Attributgruppe wartet
    
    ClientReceiverInterface empf = new ClientReceiverInterface()
    {
      public void update( ResultData[] results )
      {
        for (int i = 0; i < results.length; i++) 
        {
          ResultData dat = results [i];

          // Auslesen der Attributgruppe

          if (dat.getDataDescription().getAttributeGroup().getPid().equals( _atg1 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              Array e = dat.getData().getArray( "Prozesse" );
              
              int anzahl = e.getLength();

              for (int a = 0; a < anzahl; a++) 
              {
                String name = e.getItem( a ).getTextValue( "Name" ).getValueText();
                System.out.println("Inkarnation = " + name);
              }
            }
          }
          
          if (dat.getDataDescription().getAttributeGroup().getPid().equals( _atg2 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              Array e = dat.getData().getArray( "StartStoppBloecke" );
              
              int anzahl = e.getLength();

              if (anzahl == 1)
              {
                for (int a = 0; a < anzahl; a++) 
                {
                  String startStoppID = e.getItem( a ).getTextValue( "StartStoppID" ).getValueText();
                  Array e1 = e.getItem( a ).getArray( "Inkarnationen" );
                  
                  int anzahl1 = e1.getLength();
                  
                  if (anzahl1 == 4)
                  {
                    for (int a1=0; a1<anzahl1; ++a1)
                    {
                      String nameInkarnation = e1.getItem( a1 ).getTextValue( "ProzessID" ).getValueText();
                      System.out.println("Name Inkarnation = " + nameInkarnation);
                      
                      if (nameInkarnation.contains( "Datenverteiler" ))
                        m_pidDatenverteiler = nameInkarnation;
                    }

                    m_ok = true;
                  }
                }
              }
            }
          }
        }

        if (m_ok)
        {
          m_empfangen = true;
          
          synchronized (sync)
          {
            sync.notify();
          }
        }
      }
    };

    // Anmelden zum Empfangen der zu prfenden Attributgruppe
    
//    AttributeGroup atg1 = m_connection.getDataModel().getAttributeGroup( _atg1 );
//    Aspect asp1         = m_connection.getDataModel().getAspect("asp.zustand");

    AttributeGroup atg2 = m_connection.getDataModel().getAttributeGroup( _atg2 );
    Aspect asp2         = m_connection.getDataModel().getAspect("asp.zustand");

    String pidRechner = DaVKommunikation.getInstanz().getPidRechner();
    SystemObject sysObj = m_connection.getDataModel().getObject(pidRechner);
    
//    DataDescription dd1 = new DataDescription(atg1, asp1);
    DataDescription dd2 = new DataDescription(atg2, asp2);
  
//    m_connection.subscribeReceiver(empf, sysObj, dd1, ReceiveOptions.normal(), ReceiverRole.receiver());
    m_connection.subscribeReceiver(empf, sysObj, dd2, ReceiveOptions.normal(), ReceiverRole.receiver());
  
    // Warten auf Antwort des ClientReceiverInterface
    
    synchronized (sync)
    {
      try
      {
        sync.wait(300000);
      }
      catch (InterruptedException e)
      {
        e.printStackTrace();
      }
    }

    m_connection.unsubscribeReceiver( empf, sysObj, dd2);

    if (!m_empfangen)
      fail ("StartStoppAntwort nicht korrekt empfangen");
    
    // Kernsystem wieder beenden
    
    if (m_pidDatenverteiler != null)
    {
      sendeStartStoppAnfrageV4 ( "stoppen", m_pidDatenverteiler, "", "0" );
      
      m_connection.disconnect(false, "");
    }
    
    mySleep( 10000 );
    
    System.out.println("Ende");
  }

  /**
   * Sleep
   * @param ms Millisekunden
   */
  private void mySleep (long ms)
  {
    try
    {
      Thread.sleep (ms);
    }
    catch (InterruptedException e)
    {
      e.printStackTrace();
    } 
  }
  
  /**
   * Methode zum Senden einer StartStoppAnfrage Variante 4
   * @param aktion Aktion
   * @param prozessID ProzessID
   * @param prozessName Name des Prozesses
   * @param simulationsVariante Simulationsvariante
   */
  private void sendeStartStoppAnfrageV4 ( String aktion, String prozessID, String prozessName, String simulationsVariante )
  {
    AttributeGroup atg = (AttributeGroup) m_connection.getDataModel().getAttributeGroup("atg.startStoppAnfrage");   
    
    Data data = m_connection.createData(atg );
    
    if (data != null)
    {
      data.getReferenceValue( "absenderId" ).setText( new String("" + m_connection.getLocalApplicationObject().getId()) );
      data.getTextValue( "absenderZeichen" ).setText( new String("Absender Zeichen XYZ") );

      data.getArray( "V1" ).setLength(0);
      data.getArray( "V3" ).setLength(0);
      data.getArray( "V2" ).setLength(0);
      data.getArray( "V4" ).setLength(1);
      data.getArray( "V4" ).getItem( 0 ).getScaledValue("Aktion").setText( aktion );   
      data.getArray( "V4" ).getItem( 0 ).getTextValue( "ProzessID").setText(prozessID);
      data.getArray( "V4" ).getItem( 0 ).getTextValue( "Name").setText ( prozessName );
      
      sendeDaten (data);
    }
  }

  /**
   * Methode zum Senden der Attributgruppe atg.startStoppAnfrage an den DaV
   * @param data zu sendende Daten
   */
  private void sendeDaten (Data data)
  {
    SystemObject sysObj = m_connection.getDataModel().getObject("rechner.Rechner3");

    AttributeGroup atg = (AttributeGroup) m_connection.getDataModel().getAttributeGroup("atg.startStoppAnfrage");   
    Aspect asp         = m_connection.getDataModel().getAspect("asp.anfrage");
    DataDescription dd = new DataDescription(atg, asp);

    ClientSenderInterface send = new ClientSenderInterface () 
    {
      public void dataRequest( SystemObject object, DataDescription dataDescription, byte state )
      {
      }

      public boolean isRequestSupported( SystemObject object, DataDescription dataDescription )
      {
        return false;
      }
    };
    
    try
    {
      m_connection.subscribeSender ( send, sysObj, dd, SenderRole.sender());
    }
    catch ( OneSubscriptionPerSendData e1 )
    {
      e1.printStackTrace();
    }

    ResultData dr = new ResultData( sysObj, dd, new Date().getTime(), data );

    try
    {
      m_connection.sendData( dr );
    }
    catch ( DataNotSubscribedException e1 )
    {
      e1.printStackTrace();
    }
    catch ( SendSubscriptionNotConfirmed e1 )
    {
      e1.printStackTrace();
    }
    
    // Wieder abmelden
    
    m_connection.unsubscribeSender(send, sysObj, dd);
  }
}


