/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.sys.startstopp.skriptvew;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Klasse zum Testen der Klasse HashWert
 * @author Dambach Werke GmbH
 *
 */
public class HashWertTest
{
  /**
   * Name einer temp. Datei 
   */
  private static String m_datei        = "prozessverwaltungTest.tmp";

  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.skriptvew.Hashwert#toHexString(byte)}.
   */
  @Test
  public void testToHexString()
  {
    Hashwert hw = new Hashwert ();
    
    byte [] input = new byte [10];
    
    input [0] =          0;
    input [1] =          1;
    input [2] =          9;
    input [3] =         10;
    input [4] =         15;
    input [5] =         16;
    input [6] =        127;
    input [7] = (byte) 255;
    
    String [] erwarteterOutput = new String [10];
    
    erwarteterOutput [0] = "00";
    erwarteterOutput [1] = "01";
    erwarteterOutput [2] = "09";
    erwarteterOutput [3] = "0A";
    erwarteterOutput [4] = "0F";
    erwarteterOutput [5] = "10";
    erwarteterOutput [6] = "7F";
    erwarteterOutput [7] = "FF";

    for (int i=0; i<8; ++i)
      if (!erwarteterOutput[i].equals( hw.toHexString( input [i] ) ))
        fail ("Fehler beim Umwandeln von " + input [i] + " nach " + erwarteterOutput [i]);
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.skriptvew.Hashwert#berechneHashwert(java.lang.String, java.lang.String)}.
   */
  @Test
  public void testBerechneHashwert()
  {
    Hashwert hw = new Hashwert ();
    
    String input [] = new String [10];
    
    input [0] = "                   ";
    input [1] = "1 1 1 1 1 1 1 1 1 1";
    input [2] = " 1 1 1 1 1 1 1 1 1 ";
    input [3] = "abcdefghijklmnopqrs";
    input [4] = "abcdefghijklmnopqrt";
    
    String [] erwarteterOutput = new String [10];
    
    erwarteterOutput [0] = "7AED846F5B3DE8A31C56E49A627F5F70";
    erwarteterOutput [1] = "8FCA474DD011C5109FA50B7D5F9D6FB5";
    erwarteterOutput [2] = "F7A5BFA934DA3CC222396E4E040A4E39";
    erwarteterOutput [3] = "320620468E1C404151741BBBAFF2ADA5";
    erwarteterOutput [4] = "26596044DF08603FAC5ADD43368B116D";
    
    for (int i=0; i<5; ++i)
    {
      XMLToolsTest test = new XMLToolsTest ();
      test.schreibeInDatei( m_datei, input [i] );

      if (!erwarteterOutput [i].equals( hw.berechneHashwert( "MD5", m_datei )))
          fail ("Fehler bei der Hashwertberechnung von " + input [i] + " nach " + erwarteterOutput [i]);
    }
  }

}
